var map;
var layers = [];

var currentViewState = {
    longitude: -122.45,
    latitude: 37.78,
    zoom: 12
};

const DARK_TILE = 'https://tiles.stadiamaps.com/tiles/alidade_smooth_dark/{z}/{x}/{y}.png';

function createOrthomapLayer(props) {
    return new deck.TileLayer({
        id: 'OrthomapLayer',
        data: 'https://c.tile.openstreetmap.org/{z}/{x}/{y}.png',

        minZoom: 0,
        maxZoom: 19,
        tileSize: 256,

        renderSubLayers: props => {
            const {
                bbox: { west, south, east, north }
            } = props.tile;

            return new deck.BitmapLayer(props, {
                data: null,
                image: props.data,
                bounds: [west, south, east, north]
            });
        },
        ...props,
    });
}

function createArcLayer(props) {
    return new deck.ArcLayer({
        id: 'ArcLayer',
        data: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/bart-segments.json',

        /* props from ArcLayer class */

        // getHeight: 1,
        getSourceColor: d => [Math.sqrt(d.inbound), 140, 0],
        getSourcePosition: d => d.from.coordinates,
        getTargetColor: d => [Math.sqrt(d.outbound), 140, 0],
        getTargetPosition: d => d.to.coordinates,
        // getTilt: 0,
        getWidth: 12,
        // greatCircle: false,
        // numSegments: 50,
        // widthMaxPixels: Number.MAX_SAFE_INTEGER,
        // widthMinPixels: 0,
        // widthScale: 1,
        // widthUnits: 'pixels',

        /* props inherited from Layer class */

        // autoHighlight: false,
        // coordinateOrigin: [0, 0, 0],
        // coordinateSystem: COORDINATE_SYSTEM.LNGLAT,
        // highlightColor: [0, 0, 128, 128],
        // modelMatrix: null,
        // opacity: 1,
        pickable: true,
        // visible: true,
        // wrapLongitude: false,
        ...props
    });
}

function createHeatmapLayer(props) {
    return new deck.ColumnLayer({
        id: 'ColumnLayer',
        data: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/hexagons.json',

        /* props from ColumnLayer class */

        // angle: 0,
        // coverage: 1,
        diskResolution: 12,
        elevationScale: 100,
        extruded: true,
        // filled: true,
        getElevation: d => d.value * 50,
        getFillColor: d => [48, 128, d.value * 255, 255],
        getLineColor: [0, 0, 0],
        getLineWidth: 20,
        getPosition: d => d.centroid,
        // lineWidthMaxPixels: Number.MAX_SAFE_INTEGER,
        // lineWidthMinPixels: 0,
        // lineWidthScale: 1,
        // lineWidthUnits: 'meters',
        // material: true,
        // offset: [0, 0],
        radius: 250,
        // radiusUnits: 'meters',
        // stroked: false,
        // vertices: null,
        // wireframe: false,

        /* props inherited from Layer class */

        // autoHighlight: false,
        // coordinateOrigin: [0, 0, 0],
        // coordinateSystem: COORDINATE_SYSTEM.LNGLAT,
        // highlightColor: [0, 0, 128, 128],
        // modelMatrix: null,
        // opacity: 1,
        pickable: true,
        // visible: true,
        // wrapLongitude: false,
    });
}

function createHexagonLayer(props) {
    return new deck.HexagonLayer({
        id: 'HexagonLayer',
        data: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/sf-bike-parking.json',

        /* props from HexagonLayer class */

        // colorAggregation: 'SUM',
        // colorDomain: null,
        // colorRange: [[255, 255, 178], [254, 217, 118], [254, 178, 76], [253, 141, 60], [240, 59, 32], [189, 0, 38]],
        // colorScaleType: 'quantize',
        // coverage: 1,
        // elevationAggregation: 'SUM',
        // elevationDomain: null,
        // elevationLowerPercentile: 0,
        // elevationRange: [0, 1000],
        elevationScale: 4,
        // elevationScaleType: 'linear',
        // elevationUpperPercentile: 100,
        extruded: true,
        // getColorValue: null,
        // getColorWeight: 1,
        // getElevationValue: null,
        // getElevationWeight: 1,
        getPosition: d => d.COORDINATES,
        // hexagonAggregator: null,
        // lowerPercentile: 0,
        // material: true,
        // onSetColorDomain: null,
        // onSetElevationDomain: null,
        radius: 200,
        // upperPercentile: 100,

        /* props inherited from Layer class */

        // autoHighlight: false,
        // coordinateOrigin: [0, 0, 0],
        // coordinateSystem: COORDINATE_SYSTEM.LNGLAT,
        // highlightColor: [0, 0, 128, 128],
        // modelMatrix: null,
        // opacity: 1,
        pickable: true,
        // visible: true,
        // wrapLongitude: false,
        ...props
    });
}

function createTerrainLayer(props) {
    return new deck.TerrainLayer({
        id: 'TerrainLayer',

        /* props from TerrainLayer class */

        // TilesetClass: null,
        bounds: [-122.5233, 37.6493, -122.3566, 37.8159],
        // color: [255, 255, 255],
        // data: [],
        elevationData: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/terrain.png',
        elevationDecoder: {
            rScaler: 2,
            gScaler: 0,
            bScaler: 0,
            offset: 0
        },
        // extent: null,
        // getTileData: null,
        // loaders: ,
        material: {
            diffuse: 1
        },
        // maxCacheByteSize: null,
        // maxCacheSize: null,
        // maxRequests: 6,
        // maxZoom: null,
        // meshMaxError: 4,
        // minZoom: 0,
        // onTileError: null,
        // onTileLoad: null,
        // onTileUnload: null,
        // onViewportLoad: null,
        // refinementStrategy: 'best-available',
        // renderSubLayers: null,
        texture: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/terrain-mask.png',
        // tileSize: 512,
        // wireframe: false,
        // workerUrl: '',
        // zRange: null,
        // zoomOffset: 0,

        /* props inherited from Layer class */

        // autoHighlight: false,
        // coordinateOrigin: [0, 0, 0],
        // coordinateSystem: COORDINATE_SYSTEM.LNGLAT,
        // highlightColor: [0, 0, 128, 128],
        // modelMatrix: null,
        // opacity: 1,
        // pickable: false,
        // visible: true,
        // wrapLongitude: false,
        ...props
    });
}

function addEventListeners() {
    document.getElementById('arc_btn').addEventListener('click', () => {
        layers = [];
        layers.push(
            createOrthomapLayer(),
            createArcLayer(),
        );
        map.setProps({ layers: [...layers] });
    });
    document.getElementById('heat_btn').addEventListener('click', () => {
        layers = [];
        layers.push(
            createOrthomapLayer({ data: DARK_TILE }),
            createHeatmapLayer(),
        );
        map.setProps({ layers: [...layers] });
    });
    document.getElementById('hex_btn').addEventListener('click', () => {
        layers = [];
        layers.push(
            createOrthomapLayer(),
            createHexagonLayer(),
        );
        map.setProps({ layers: [...layers] });
    });
    document.getElementById('ter_btn').addEventListener('click', () => {
        layers = [];
        layers.push(
            createOrthomapLayer(),
            createTerrainLayer(),
        );
        map.setProps({ layers: [...layers] });
    });
}

function onLoad() {
    console.log('loading map');
    layers.push(createOrthomapLayer({ data: DARK_TILE }));
    map = new deck.Deck({
        viewState: currentViewState,
        canvas: document.getElementById('cvs'),
        onViewStateChange: ({ viewState }) => {
            currentViewState = viewState
            map.setProps({ viewState: { ...currentViewState } });
        },
        controller: true,
        layers: [
            ...layers,
        ],
        getTooltip: ({ layer, object }) => {
            if (layer) {
                switch (layer.id) {
                    case "ArcLayer":
                        return object &&
                            `${object.from.name} to ${object.to.name}`
                    case "HexagonLayer":
                        return object &&
                            `${object.position.join(', ')} 
                                Count: ${object.points.length}`
                }
            }
        }
    });
    addEventListeners();
}

window.onload = onLoad;